@@ -1,3 +1,4 @@
+import functools
 import logging
 import os
 import string
@@ -11,9 +12,9 @@ from .util import lower_dict
 
 __all__ = (
     "COMMON_STEAM_DIRS", "SteamApp", "find_steam_path",
-    "find_steam_proton_app", "find_proton_app", "find_steam_runtime_path",
-    "find_appid_proton_prefix", "get_steam_lib_paths", "get_steam_apps",
-    "get_custom_proton_installations"
+    "find_steam_proton_app", "find_proton_app",
+    "find_legacy_steam_runtime_path", "find_appid_proton_prefix",
+    "get_steam_lib_paths", "get_steam_apps", "get_custom_proton_installations"
 )
 
 COMMON_STEAM_DIRS = [
@@ -118,6 +119,34 @@ class SteamApp(object):
         """
         return (self.install_path / "toolmanifest.vdf").is_file()
 
+    @property
+    def proton_dist_path(self):
+        """
+        Return path to the directory containing Proton binaries and libraries.
+        None is returned if this app isn't a Proton installation or either
+        directory doesn't exist.
+
+        The directory is named either 'dist' or 'files'.
+
+        'dist' is used by older Proton releases, and it is extracted from a
+        separate 'proton_dist.tar' archive during first launch.
+        'files' is used by newer Proton releases, and it already exists
+        after the Steam app has been installed, requiring no first launch.
+        """
+        if not self.is_proton:
+            return None
+
+        try:
+            # Prioritize 'files' directory if it exists.
+            # If both directories exist, 'dist' is likely a leftover that
+            # wasn't removed by Steam.
+            return next(
+                (self.install_path / name) for name in ("files", "dist")
+                if (self.install_path / name).is_dir()
+            )
+        except StopIteration:
+            return None
+
     @classmethod
     def from_appmanifest(cls, path, steam_lib_paths):
         """
@@ -254,9 +283,9 @@ def find_steam_path():
     return None, None
 
 
-def find_steam_runtime_path(steam_root):
+def find_legacy_steam_runtime_path(steam_root):
     """
-    Find the Steam Runtime either using the STEAM_RUNTIME env or
+    Find the legacy Steam Runtime either using the STEAM_RUNTIME env or
     steam_root
     """
     env_steam_runtime = os.environ.get("STEAM_RUNTIME", "")
