@@ -25,10 +25,29 @@ def lower_dict(d):
     return {k.lower(): v for k, v in d.items()}
 
 
+def get_legacy_runtime_library_paths(legacy_steam_runtime_path, proton_app):
+    """
+    Get LD_LIBRARY_PATH value to use when running a command using Steam Runtime
+    """
+    steam_runtime_paths = check_output([
+        str(legacy_steam_runtime_path / "run.sh"),
+        "--print-steam-runtime-library-paths"
+    ])
+    steam_runtime_paths = str(steam_runtime_paths, "utf-8")
+    # Add Proton installation directory first into LD_LIBRARY_PATH
+    # so that libwine.so.1 is picked up correctly (see issue #3)
+    return "".join([
+        str(proton_app.proton_dist_path / "lib"), os.pathsep,
+        str(proton_app.proton_dist_path / "lib64"), os.pathsep,
+        steam_runtime_paths
+    ])
+
+
 def get_host_library_paths():
     """
     Get host library paths to use when creating the LD_LIBRARY_PATH environment
-    variable for use with newer Steam Runtime installations
+    variable for use with newer Steam Runtime installations when *not*
+    using bwrap
     """
     # The traditional Steam Runtime does the following when running the
     # `run.sh --print-steam-runtime-library-paths` command.
@@ -53,9 +72,9 @@ RUNTIME_ROOT_GLOB_PATTERNS = (
 )
 
 
-def get_runtime_library_paths(steam_runtime_path, proton_app):
+def get_runtime_library_paths(proton_app, use_bwrap=True):
     """
-    Get LD_LIBRARY_PATH value to run a command using Steam Runtime
+    Get LD_LIBRARY_PATH value to use when running a command using Steam Runtime
     """
     def find_runtime_app_root(runtime_app):
         """
@@ -76,52 +95,62 @@ def get_runtime_library_paths(steam_runtime_path, proton_app):
             )
         )
 
-    if proton_app.required_tool_appid:
-        # bwrap based Steam Runtime is used for Proton installations that
-        # use separate Steam runtimes
-        # TODO: Try to run the Wine binaries inside an user namespace somehow.
-        # Newer Steam Runtime environments may rely on a newer glibc than what
-        # is available on the host system, which may cause potential problems
-        # otherwise.
-        runtime_root = find_runtime_app_root(proton_app.required_tool_app)
+    if use_bwrap:
         return "".join([
-            str(proton_app.install_path / "dist" / "lib"), os.pathsep,
-            str(proton_app.install_path / "dist" / "lib64"), os.pathsep,
-            get_host_library_paths(), os.pathsep,
-            str(runtime_root / "lib" / "i386-linux-gnu"), os.pathsep,
-            str(runtime_root / "lib" / "x86_64-linux-gnu")
+            str(proton_app.proton_dist_path / "lib"), os.pathsep,
+            str(proton_app.proton_dist_path / "lib64"), os.pathsep
         ])
 
-    # Traditional LD_LIBRARY_PATH based Steam Runtime is used otherwise
-    steam_runtime_paths = check_output([
-        str(steam_runtime_path / "run.sh"),
-        "--print-steam-runtime-library-paths"
-    ])
-    steam_runtime_paths = str(steam_runtime_paths, "utf-8")
-    # Add Proton installation directory first into LD_LIBRARY_PATH
-    # so that libwine.so.1 is picked up correctly (see issue #3)
+    runtime_root = find_runtime_app_root(proton_app.required_tool_app)
     return "".join([
-        str(proton_app.install_path / "dist" / "lib"), os.pathsep,
-        str(proton_app.install_path / "dist" / "lib64"), os.pathsep,
-        steam_runtime_paths
+        str(proton_app.proton_dist_path / "lib"), os.pathsep,
+        str(proton_app.proton_dist_path / "lib64"), os.pathsep,
+        get_host_library_paths(), os.pathsep,
+        str(runtime_root / "lib" / "i386-linux-gnu"), os.pathsep,
+        str(runtime_root / "lib" / "x86_64-linux-gnu")
     ])
 
 
-WINE_SCRIPT_TEMPLATE = (
+WINE_SCRIPT_RUNTIME_V1_TEMPLATE = (
     "#!/bin/bash\n"
     "# Helper script created by Protontricks to run Wine binaries using Steam Runtime\n"
     "export LD_LIBRARY_PATH=\"$PROTON_LD_LIBRARY_PATH\"\n"
-    "exec \"$PROTON_PATH\"/dist/bin/{name} \"$@\""
+    "exec \"$PROTON_DIST_PATH\"/bin/{name} \"$@\""
 )
 
+WINE_SCRIPT_RUNTIME_V2_TEMPLATE = """#!/bin/bash
+# Helper script created by Protontricks to run Wine binaries using Steam Runtime
+PROTONTRICKS_PROXY_SCRIPT_PATH="{script_path}"
+if [[ -n "$PROTONTRICKS_INSIDE_STEAM_RUNTIME" ]]; then
+  # Command is being executed inside Steam Runtime
+  # LD_LIBRARY_PATH can now be set.
+  export LD_LIBRARY_PATH="$LD_LIBRARY_PATH":"$PROTON_LD_LIBRARY_PATH"
+  "$PROTON_DIST_PATH"/bin/{name} "$@"
+else
+  exec "$STEAM_RUNTIME_PATH"/run --share-pid --batch --filesystem=/mnt \
+  --filesystem=/tmp --filesystem=/run/media --filesystem=/etc \
+  --filesystem=/opt --filesystem=/home --filesystem=/usr -- \
+  env PROTONTRICKS_INSIDE_STEAM_RUNTIME=1 \
+  "$PROTONTRICKS_PROXY_SCRIPT_PATH" "$@"
+fi
+"""
 
-def create_wine_bin_dir(proton_app):
+
+def create_wine_bin_dir(proton_app, use_bwrap=True):
     """
     Create a directory with "proxy" executables that load shared libraries
     using Steam Runtime and Proton's own libraries instead of the system
     libraries
     """
-    binaries = list((proton_app.install_path / "dist" / "bin").iterdir())
+    # If the Proton installation uses a newer version of Steam Runtime,
+    # use a different template for the scripts
+    bin_template = (
+        WINE_SCRIPT_RUNTIME_V2_TEMPLATE
+        if proton_app.required_tool_app and use_bwrap
+        else WINE_SCRIPT_RUNTIME_V1_TEMPLATE
+    )
+
+    binaries = list((proton_app.proton_dist_path / "bin").iterdir())
 
     # Create the base directory containing files for every Proton installation
     xdg_cache_dir = os.environ.get(
@@ -139,16 +168,6 @@ def create_wine_bin_dir(proton_app):
         "Created Steam Runtime Wine binary directory at %s", str(bin_path)
     )
 
-    # Check if the correct binaries exist
-    files_already_exist = (
-        {binary.name for binary in binaries}
-        == {binary.name for binary in bin_path.iterdir()}
-    )
-
-    if files_already_exist:
-        # The correct files exist and nothing needs to be rewritten
-        return bin_path
-
     # Delete the directory and rewrite the scripts. Some binaries may no
     # longer exist in the Proton installation, so we'll also get rid of
     # scripts that point to non-existing files
@@ -156,11 +175,13 @@ def create_wine_bin_dir(proton_app):
     bin_path.mkdir(parents=True)
 
     for binary in binaries:
-        content = WINE_SCRIPT_TEMPLATE.format(
-            name=shlex.quote(binary.name)
+        proxy_script_path = bin_path / binary.name
+
+        content = bin_template.format(
+            name=shlex.quote(binary.name),
+            script_path=str(proxy_script_path)
         ).encode("utf-8")
 
-        proxy_script_path = bin_path / binary.name
         proxy_script_path.write_bytes(content)
 
         script_stat = proxy_script_path.stat()
@@ -172,7 +193,9 @@ def create_wine_bin_dir(proton_app):
 
 def run_command(
         winetricks_path, proton_app, steam_app, command,
-        steam_runtime_path=None,
+        use_steam_runtime=False,
+        legacy_steam_runtime_path=None,
+        use_bwrap=True,
         **kwargs):
     """Run an arbitrary command with the correct environment variables
     for the given Proton app
@@ -180,13 +203,18 @@ def run_command(
     The environment variables are set for the duration of the call
     and restored afterwards
 
-    If 'steam_runtime_path' is provided, run the command using Steam Runtime
+    If 'use_steam_runtime' is True, run the command using Steam Runtime
+    using either 'legacy_steam_runtime_path' or the Proton app's specific
+    Steam Runtime installation, depending on which one is required.
+
+    If 'use_bwrap' is True, run newer Steam Runtime installations using
+    bwrap based containerization.
     """
     # Check for incomplete Steam Runtime installation
     runtime_install_incomplete = \
         proton_app.required_tool_appid and not proton_app.required_tool_app
 
-    if steam_runtime_path and runtime_install_incomplete:
+    if use_steam_runtime and runtime_install_incomplete:
         raise RuntimeError(
             "{} is missing the required Steam Runtime. You may need to launch "
             "a Steam app using this Proton version to finish the "
@@ -196,69 +224,101 @@ def run_command(
     # Make a copy of the environment variables to restore later
     environ_copy = os.environ.copy()
 
-    if not os.environ.get("WINE"):
+    user_provided_wine = os.environ.get("WINE", False)
+    user_provided_wineserver = os.environ.get("WINESERVER", False)
+
+    if not user_provided_wine:
         logger.info(
             "WINE environment variable is not available. "
             "Setting WINE environment variable to Proton bundled version"
         )
         os.environ["WINE"] = \
-            str(proton_app.install_path / "dist" / "bin" / "wine")
+            str(proton_app.proton_dist_path / "bin" / "wine")
 
-    if not os.environ.get("WINESERVER"):
+    if not user_provided_wineserver:
         logger.info(
             "WINESERVER environment variable is not available. "
             "Setting WINESERVER environment variable to Proton bundled version"
         )
         os.environ["WINESERVER"] = \
-            str(proton_app.install_path / "dist" / "bin" / "wineserver")
+            str(proton_app.proton_dist_path / "bin" / "wineserver")
 
     os.environ["WINETRICKS"] = str(winetricks_path)
     os.environ["WINEPREFIX"] = str(steam_app.prefix_path)
     os.environ["WINELOADER"] = os.environ["WINE"]
     os.environ["WINEDLLPATH"] = "".join([
-        str(proton_app.install_path / "dist" / "lib64" / "wine"),
+        str(proton_app.proton_dist_path / "lib64" / "wine"),
         os.pathsep,
-        str(proton_app.install_path / "dist" / "lib" / "wine")
+        str(proton_app.proton_dist_path / "lib" / "wine")
     ])
 
     os.environ["PATH"] = "".join([
-        str(proton_app.install_path / "dist" / "bin"), os.pathsep,
+        str(proton_app.proton_dist_path / "bin"), os.pathsep,
         os.environ["PATH"]
     ])
 
     # Expose the path to Proton installation. This is mainly used for
     # Wine helper scripts, but other scripts could use it as well.
     os.environ["PROTON_PATH"] = str(proton_app.install_path)
+    os.environ["PROTON_DIST_PATH"] = str(proton_app.proton_dist_path)
 
     # Unset WINEARCH, which might be set for another Wine installation
     os.environ.pop("WINEARCH", "")
 
     wine_bin_dir = None
-    if steam_runtime_path:
+    if use_steam_runtime:
         if proton_app.required_tool_app:
+            os.environ["STEAM_RUNTIME_PATH"] = \
+                str(proton_app.required_tool_app.install_path)
+            os.environ["PROTON_LD_LIBRARY_PATH"] = \
+                get_runtime_library_paths(proton_app, use_bwrap=use_bwrap)
+
             runtime_name = proton_app.required_tool_app.name
             logger.info(
                 "Using separately installed Steam Runtime: %s",
                 runtime_name
             )
 
+            if use_bwrap:
+                logger.warning(
+                    "Running Steam Runtime using bwrap containerization. "
+                    "This is an experimental feature and it may still be "
+                    "incompatible in some situations.\n"
+                    "If any problems arise, please try running the command "
+                    "again using the `--no-bwrap` flag and make an issue "
+                    "report if the problem only occurs when bwrap is in use."
+                )
+
             if runtime_name not in SUPPORTED_STEAM_RUNTIMES:
                 logger.warning(
                     "Current Steam Runtime not recognized by Protontricks."
                 )
+        else:
+            # Legacy Steam Runtime requires a different LD_LIBRARY_PATH
+            os.environ["PROTON_LD_LIBRARY_PATH"] = \
+                get_legacy_runtime_library_paths(
+                    legacy_steam_runtime_path, proton_app
+                )
 
         # When Steam Runtime is enabled, create a set of helper scripts
         # that load the underlying Proton Wine executables with Steam Runtime
         # and Proton libraries instead of system libraries
-        wine_bin_dir = create_wine_bin_dir(proton_app=proton_app)
-        os.environ["PROTON_LD_LIBRARY_PATH"] = \
-            get_runtime_library_paths(steam_runtime_path, proton_app)
+        wine_bin_dir = create_wine_bin_dir(
+            proton_app=proton_app, use_bwrap=use_bwrap
+        )
+        os.environ["LEGACY_STEAM_RUNTIME_PATH"] = \
+            str(legacy_steam_runtime_path)
+
         os.environ["PATH"] = "".join([
             str(wine_bin_dir), os.pathsep, os.environ["PATH"]
         ])
-        os.environ["WINE"] = str(wine_bin_dir / "wine")
-        os.environ["WINELOADER"] = os.environ["WINE"]
-        os.environ["WINESERVER"] = str(wine_bin_dir / "wineserver")
+
+        if not user_provided_wine:
+            os.environ["WINE"] = str(wine_bin_dir / "wine")
+            os.environ["WINELOADER"] = os.environ["WINE"]
+
+        if not user_provided_wineserver:
+            os.environ["WINESERVER"] = str(wine_bin_dir / "wineserver")
 
     logger.info("Attempting to run command %s", command)
 
